#!/bin/bash
#
# ssh-api:rsync-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo "ERROR in '${BASH_SOURCE[0]}' - project https://git.cclaude.rocks/bash-apis/bash-common-api.git is required." >&2
  echo "ERROR in '${BASH_SOURCE[0]}' - source message-api.sh" >&2
  exit 1
fi

if [ "$( type -t ssh_api_test_remote_directory )" != 'function' ] ; then
  # shellcheck disable=SC1091
  source "$( dirname "${BASH_SOURCE[0]}" )/ssh-common-api.sh" || exit $?
fi

function ssh_api_rsync_directory_from_host {
  local -r _ssh_user_="$1"
  local -r _ssh_host_="$2"
  local -r _ssh_remote_dir_="$3"
  local -r _local_directory_="$4"

  local _ssh_ip_host_=
  _ssh_ip_host_="$( _ssh_api_common_get_ssh_host_ip_ "${_ssh_host_}" )" || return $?
  local -r _ssh_ip_host_

  if [ "$( getRSYNC_CHECK_DESTINATION )" != 'false' ] ; then
    local result=
    result="$( ssh_api_test_remote_directory "${_ssh_user_}" "${_ssh_ip_host_}" "${_ssh_remote_dir_}" )"
    local -ri ssh_rc=$?

    if [ "${result}" != 'FOUND' ] ; then
      echofatal "${FUNCNAME[0]}() - Directory '${_ssh_remote_dir_}' not found on host '${_ssh_ip_host_}'"
      echofatal "rc: ${ssh_rc} - '${result}'"
      if [ "${result}" = 'NOTEXISTS' ] ; then
        echofatal "${FUNCNAME[0]}() - Directory '${_ssh_remote_dir_}' does not exist on host '${_ssh_ip_host_}'"
      else
        echofatal "${FUNCNAME[0]}() - ssh error with ${ssh_rc} running with '${USER:-$( id -u -n )}' from: '$( pwd )'"
        echofatal "${FUNCNAME[0]}() - USER: '${USER:-}' / id -u -n: '$( id -u -n )'"
        echofatal "${FUNCNAME[0]}() - remote user '${_ssh_user_}' / remote host '${_ssh_ip_host_}'"
      fi
      return 100
    fi
  fi

  local _rsync_opts_=
  _rsync_opts_="$( getRSYNC_OPTS )" || return $?
  local -r _rsync_opts_

  (
    if ssh_api_is_SSH_API_RSYNC_DEBUG ; then
      set -x
    fi

    # shellcheck disable=SC2086
    command rsync ${_rsync_opts_} \
        "${_ssh_user_}@${_ssh_ip_host_}:${_ssh_remote_dir_}" \
        "${_local_directory_}/"
  ) || return $?
}

function ssh_api_rsync_directory_locally {
  local -r _local_directory_="$1"
  local -r _ssh_remote_dir_="$2"

  if [ ! -d "${_local_directory_}" ] ; then
    echofatal "${FUNCNAME[0]}() - Directory '${_local_directory_}' not found."
    return 100
  fi

  if [ "$( getRSYNC_CHECK_DESTINATION )" != 'false' ] ; then
    if [ ! -d "${_ssh_remote_dir_}" ] ; then
      echofatal "${FUNCNAME[0]}() - Directory '${_ssh_remote_dir_}' not found."
      return 100
    fi
  fi

  local _rsync_opts_=
  _rsync_opts_="$( getRSYNC_OPTS )" || return $?
  local -r _rsync_opts_

  (
    if ssh_api_is_SSH_API_RSYNC_DEBUG ; then
      set -x
    fi

    # shellcheck disable=SC2086
    command rsync ${_rsync_opts_} "${_local_directory_}/" "${_ssh_remote_dir_}"
  ) || return $?
}

function ssh_api_rsync_directory_to_host {
  local -r _local_directory_="$1"
  local -r _ssh_user_="$2"
  local -r _ssh_host_="$3"
  local -r _ssh_remote_dir_="$4"

  if [ ! -d "${_local_directory_}" ] ; then
    echofatal "${FUNCNAME[0]}() - Directory '${_local_directory_}' not found."
    return 100
  fi

  local _ssh_ip_host_=
  _ssh_ip_host_="$( _ssh_api_common_get_ssh_host_ip_ "${_ssh_host_}" )" || return $?
  local -r _ssh_ip_host_

  if [ "$( getRSYNC_CHECK_DESTINATION )" != 'false' ] ; then
    local result=
    result="$( ssh_api_test_remote_directory "${_ssh_user_}" "${_ssh_ip_host_}" "${_ssh_remote_dir_}" )"
    local -ri ssh_rc=$?

    if [ "${result}" != 'FOUND' ] ; then
      echofatal "${FUNCNAME[0]}() - Directory '${_ssh_remote_dir_}' not found on host '${_ssh_ip_host_}'"
      echofatal "rc: ${ssh_rc} - '${result}'"
      if [ "${result}" = 'NOTEXISTS' ] ; then
        echofatal "${FUNCNAME[0]}() - Directory '${_ssh_remote_dir_}' does not exist on host '${_ssh_ip_host_}'"
      else
        echofatal "${FUNCNAME[0]}() - ssh error with ${ssh_rc} running with '${USER:-$( id -u -n )}' from: '$( pwd )'"
        echofatal "${FUNCNAME[0]}() - USER: '${USER:-}' / id -u -n: '$( id -u -n )'"
        echofatal "${FUNCNAME[0]}() - remote user '${_ssh_user_}' / remote host '${_ssh_ip_host_}'"
      fi
      return 100
    fi
  fi

  local _rsync_opts_=
  _rsync_opts_="$( getRSYNC_OPTS )" || return $?
  local -r _rsync_opts_

  (
    if ssh_api_is_SSH_API_RSYNC_DEBUG ; then
      set -x
    fi

    # shellcheck disable=SC2086
    command rsync ${_rsync_opts_} \
        "${_local_directory_}/" \
        "${_ssh_user_}@${_ssh_ip_host_}:${_ssh_remote_dir_}"
  ) || return $?
}

function ssh_api_rsync_file_from_host {
  local -r _ssh_user_="$1"
  local -r _ssh_host_="$2"
  local -r _ssh_remote_file_="$3"
  local -r _local_directory_="$4"

  local _ssh_ip_host_=
  _ssh_ip_host_="$( _ssh_api_common_get_ssh_host_ip_ "${_ssh_host_}" )" || return $?
  local -r _ssh_ip_host_

  local _rsync_opts_=
  _rsync_opts_="$( getRSYNC_OPTS )" || return $?
  local -r _rsync_opts_

  (
    if ssh_api_is_SSH_API_RSYNC_DEBUG ; then
      set -x
    fi

    # shellcheck disable=SC2086
    command rsync ${_rsync_opts_} \
        "${_ssh_user_}@${_ssh_ip_host_}:${_ssh_remote_file_}" \
        "${_local_directory_}/"
  ) || return $?
}

